----------------------------------------------------------------------------------
-- Company: Univ Lyon, UJM-Saint-Etienne, CNRS, Laboratoire Hubert Curien UMR 5516, F-42023, SAINT-ETIENNE, France
-- Creator: Cédric Marchand 
-- 
-- Create Date:    18:07:09 11/11/2015 
-- Design Name: 
-- Module Name:    Datapath - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

LIBRARY work;
USE work.API_pkg.ALL;

entity Datapath is
  PORT(
    clk_i : in std_logic;
    --Input
    data_i    : in std_logic_vector(7 downto 0);
    key_i     : in std_logic_vector(7 downto 0);
    --Control
    mode_i    : in std_logic;
    key_sel   : in std_logic_vector(1 downto 0);
    data_sel  : in std_logic_vector(1 downto 0);
    count_r   : in std_logic_vector(4 downto 0);
    --Output
    data_o : out std_logic_vector(7 downto 0)
  );
end Datapath;

architecture Behavioral of Datapath is

COMPONENT keyschedule_v2
	PORT(
		key_in : IN std_logic_vector(79 downto 0);
		mode_i : IN std_logic;          
		key_o  : OUT std_logic_vector(79 downto 0)
		);
	END COMPONENT;
  
  COMPONENT extract_key
	PORT(
		key_in : IN std_logic_vector(79 downto 0);
		i      : IN std_logic_vector(4 downto 0);          
		rk     : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;
  
  COMPONENT round is
	port (
		rk			: in std_logic_vector(31 downto 0);
		round_i	: in std_logic_vector(63 downto 0);
		round_o	: out std_logic_vector(63 downto 0)
	);
  END COMPONENT;

  COMPONENT permutation_layer
	PORT(
		mode_i  : IN std_logic;
		state_i : IN std_logic_vector(63 downto 0);          
		state_o : OUT std_logic_vector(63 downto 0)
		);
	END COMPONENT;
  
  --signals
  --regsiters
  signal key_reg    : std_logic_vector(key_size-1 downto 0);
  signal state_reg  : std_logic_vector(state_size-1 downto 0);
  
  signal rk_s             : std_logic_vector(31 downto 0);
  signal round_s        : std_logic_vector(state_size-1 downto 0);
  signal perm_s         : std_logic_vector(state_size-1 downto 0);
  signal k_s_o         : std_logic_vector(key_size-1 downto 0);
  
begin
  --key process
  K_proc : process(clk_i, key_sel)
  begin
    if rising_edge(clk_i) then
      if key_sel = "01" then
        key_reg <= key_i & key_reg(key_size-1 downto 8);
      elsif key_sel = "10" then
		  key_reg <= k_s_o;
      else
        key_reg <= key_reg;
      end if;
    end if;
  end process;
  
  --data process
  D_proc : process(clk_i, data_sel)
  begin
    if rising_edge(clk_i) then
      if data_sel = "01" then
        state_reg <= data_i & state_reg(state_size-1 downto 8);
      elsif data_sel = "10" then
        state_reg <= round_s;
      elsif data_sel = "11" then
        state_reg <= perm_s;
      else
        state_reg <= state_reg;
      end if;
    end if;
  end process;
  
  data_o <= state_reg(7 downto 0); --when done_s = '1' else x"00";

  -- cipher component
  --keyschedule
  Inst_keyschedule: keyschedule_v2 PORT MAP(
		key_in => key_reg,
		mode_i => mode_i,
		key_o  => k_s_o
	);
  
  Inst_extract_key: extract_key PORT MAP(
		key_in => key_reg,
		i => count_r,
		rk => rk_s
	);
  
  Inst_round: round PORT MAP(
		rk => rk_s,
		round_i => state_reg,
		round_o => round_s
	);
  
  --permutation layer
  Inst_permutation_layer: permutation_layer PORT MAP(
		mode_i  => mode_i,
		state_i => round_s,
		state_o => perm_s
	);
  
end Behavioral;

